@@ -322,7 +322,7 @@ impl Airtable {
         // Build the request.
         let request = self.request(
             Method::GET,
-            format!("meta/enterpriseAccounts/{}/users", self.enterprise_account_id),
+            format!("v0/meta/enterpriseAccounts/{}/users", self.enterprise_account_id),
             (),
             Some(vec![("state", "provisioned".to_string())]),
         );
@@ -344,6 +344,94 @@ impl Airtable {
         Ok(result.users)
     }
 
+    /// Get an enterprise user.
+    /// This is for an enterprise admin to do only.
+    /// FROM: https://airtable.com/api/enterprise#enterpriseAccountUserGetInformationByEmail
+    /// Permission level can be: owner | create | edit | comment | read
+    pub async fn get_enterprise_user(&self, email: &str) -> Result<EnterpriseUser, APIError> {
+        if self.enterprise_account_id.is_empty() {
+            // Return an error early.
+            return Err(APIError {
+                status_code: StatusCode::OK,
+                body: "An enterprise account id is required.".to_string(),
+            });
+        }
+
+        // Build the request.
+        let request = self.request(
+            Method::GET,
+            format!("v0/meta/enterpriseAccounts/{}/users", self.enterprise_account_id),
+            (),
+            Some(vec![("email", email.to_string()), ("include", "collaborations".to_string())]),
+        );
+
+        let resp = self.client.execute(request).await.unwrap();
+        match resp.status() {
+            StatusCode::OK => (),
+            s => {
+                return Err(APIError {
+                    status_code: s,
+                    body: resp.text().await.unwrap(),
+                })
+            }
+        };
+
+        let r: EnterpriseUsersResponse = resp.json().await.unwrap();
+
+        if r.users.is_empty() {
+            return Err(APIError {
+                status_code: StatusCode::NOT_FOUND,
+                body: format!("Could not find user with email: {}", email),
+            });
+        }
+
+        Ok(r.users.get(0).unwrap().clone())
+    }
+
+    /// Add a collaborator to a workspace.
+    /// This is for an enterprise admin to do only.
+    /// FROM: https://airtable.com/api/enterprise#enterpriseWorkspaceAddCollaborator
+    /// Permission level can be: owner | create | edit | comment | read
+    pub async fn add_collaborator_to_workspace(&self, workspace_id: &str, user_id: &str, permission_level: &str) -> Result<(), APIError> {
+        if self.enterprise_account_id.is_empty() {
+            // Return an error early.
+            return Err(APIError {
+                status_code: StatusCode::OK,
+                body: "An enterprise account id is required.".to_string(),
+            });
+        }
+
+        // Build the request.
+        let request = self.request(
+            Method::POST,
+            format!("v0/meta/workspaces/{}/collaborators", workspace_id),
+            NewCollaborator {
+                collaborators: vec![Collaborator {
+                    user: User {
+                        id: user_id.to_string(),
+                        email: Default::default(),
+                        name: Default::default(),
+                    },
+                    permission_level: permission_level.to_string(),
+                }],
+            },
+            None,
+        );
+
+        let resp = self.client.execute(request).await.unwrap();
+        match resp.status() {
+            StatusCode::OK => (),
+            s => {
+                return Err(APIError {
+                    status_code: s,
+                    body: resp.text().await.unwrap(),
+                })
+            }
+        };
+
+        Ok(())
+    }
+
     /// Delete internal user by email.
     /// This is for an enterprise admin to do only.
     /// The user must be an internal user, meaning they have an email with the company domain.
@@ -360,7 +448,7 @@ impl Airtable {
         // Build the request.
         let request = self.request(
             Method::DELETE,
-            format!("meta/enterpriseAccounts/{}/users", self.enterprise_account_id),
+            format!("v0/meta/enterpriseAccounts/{}/users", self.enterprise_account_id),
             (),
             Some(vec![("email", email.to_string())]),
         );
@@ -596,6 +684,12 @@ pub struct ErrorResponse {
     pub message: String,
 }
 
+#[derive(Debug, Default, Clone, Serialize, Deserialize)]
+pub struct AttachmentShort {
+    #[serde(default, skip_serializing_if = "String::is_empty")]
+    pub url: String,
+}
+
 #[derive(Debug, Default, Clone, Serialize, Deserialize)]
 pub struct Attachment {
     #[serde(default, skip_serializing_if = "String::is_empty")]
@@ -632,6 +726,73 @@ pub struct Full {
     pub height: i64,
 }
 
+#[derive(Debug, Default, Clone, Serialize, Deserialize)]
+pub struct NewCollaborator {
+    #[serde(default, skip_serializing_if = "Vec::is_empty")]
+    pub collaborators: Vec<Collaborator>,
+}
+
+#[derive(Debug, Default, Clone, Serialize, Deserialize)]
+pub struct Collaborator {
+    #[serde(default)]
+    pub user: User,
+    #[serde(default, skip_serializing_if = "String::is_empty", deserialize_with = "deserialize_null_string::deserialize", rename = "permissionLevel")]
+    pub permission_level: String,
+}
+
+#[derive(Debug, Default, Clone, Serialize, Deserialize)]
+pub struct EnterpriseUsersResponse {
+    #[serde(default, skip_serializing_if = "Vec::is_empty")]
+    pub users: Vec<EnterpriseUser>,
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct EnterpriseUser {
+    #[serde(default, skip_serializing_if = "String::is_empty", deserialize_with = "deserialize_null_string::deserialize")]
+    pub id: String,
+    #[serde(default, skip_serializing_if = "String::is_empty", deserialize_with = "deserialize_null_string::deserialize")]
+    pub state: String,
+    #[serde(default, skip_serializing_if = "String::is_empty", deserialize_with = "deserialize_null_string::deserialize")]
+    pub email: String,
+    #[serde(default, skip_serializing_if = "String::is_empty", deserialize_with = "deserialize_null_string::deserialize")]
+    pub name: String,
+    #[serde(default, skip_serializing_if = "Option::is_none", rename = "lastActivityTime")]
+    pub last_activity_time: Option<DateTime<Utc>>,
+    #[serde(
+        default,
+        skip_serializing_if = "String::is_empty",
+        deserialize_with = "deserialize_null_string::deserialize",
+        rename = "invitedToAirtableByUserId"
+    )]
+    pub invited_to_airtable_by_user_id: String,
+    #[serde(rename = "createdTime")]
+    pub created_time: DateTime<Utc>,
+    #[serde(default)]
+    pub collaborations: Collaborations,
+}
+
+#[derive(Debug, Default, Clone, Serialize, Deserialize)]
+pub struct Collaborations {
+    #[serde(default, skip_serializing_if = "Vec::is_empty", rename = "workspaceCollaborations")]
+    pub workspace_collaborations: Vec<Collaboration>,
+    #[serde(default, skip_serializing_if = "Vec::is_empty", rename = "baseCollaborations")]
+    pub base_collaborations: Vec<Collaboration>,
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct Collaboration {
+    #[serde(default, skip_serializing_if = "String::is_empty", deserialize_with = "deserialize_null_string::deserialize", rename = "baseId")]
+    pub base_id: String,
+    #[serde(default, skip_serializing_if = "String::is_empty", deserialize_with = "deserialize_null_string::deserialize", rename = "permissionLevel")]
+    pub permission_level: String,
+    #[serde(rename = "createdTime")]
+    pub created_time: DateTime<Utc>,
+    #[serde(default, skip_serializing_if = "String::is_empty", deserialize_with = "deserialize_null_string::deserialize", rename = "grantedByUserId")]
+    pub granted_by_user_id: String,
+    #[serde(default, skip_serializing_if = "String::is_empty", deserialize_with = "deserialize_null_string::deserialize", rename = "workspaceId")]
+    pub workspace_id: String,
+}
+
 struct AttachmentsVisitor;
 
 impl<'de> Visitor<'de> for AttachmentsVisitor {
@@ -743,7 +904,7 @@ pub mod user_format_as_string {
 }
 
 pub mod attachment_format_as_array_of_strings {
-    use super::{Attachment, AttachmentsVisitor};
+    use super::{AttachmentShort, AttachmentsVisitor};
     use serde::ser::SerializeSeq;
     use serde::{self, Deserializer, Serializer};
 
@@ -761,7 +922,7 @@ pub mod attachment_format_as_array_of_strings {
         // Make our array of Airtable attachment objects.
         let mut seq = serializer.serialize_seq(Some(array.len())).unwrap();
         for e in array {
-            let mut attachment: Attachment = Default::default();
+            let mut attachment: AttachmentShort = Default::default();
             attachment.url = e.to_string();
             seq.serialize_element(&attachment).unwrap();
         }
@@ -964,3 +1125,23 @@ pub mod barcode_format_as_string {
         Ok(barcode.text)
     }
 }
+
+pub mod deserialize_null_string {
+    use serde::{self, Deserialize, Deserializer};
+
+    // The signature of a deserialize_with function must follow the pattern:
+    //
+    //    fn deserialize<'de, D>(D) -> Result<T, D::Error>
+    //    where
+    //        D: Deserializer<'de>
+    //
+    // although it may also be generic over the output types T.
+    pub fn deserialize<'de, D>(deserializer: D) -> Result<String, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        let s = String::deserialize(deserializer).unwrap_or_default();
+
+        Ok(s)
+    }
+}
